package djmil.cordacheckers.cordaclient;

import static java.util.Objects.requireNonNull;

import java.util.List;

import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import djmil.cordacheckers.cordaclient.dao.HoldingIdentity;
import djmil.cordacheckers.cordaclient.dao.VirtualNode;
import djmil.cordacheckers.cordaclient.dao.VirtualNodeList;
import djmil.cordacheckers.cordaclient.dao.flow.RequestBody;
import djmil.cordacheckers.cordaclient.dao.flow.arguments.Empty;

@Service
public class CordaClient {
    private final RestTemplate restTemplate;

    public CordaClient(RestTemplate restTemplate) {
        //System.out.println("Creating REST Service");
        // this.restTemplate = restTemplateBuilder
        //     .basicAuthentication("admin", "admin")
        //     .build();
        this.restTemplate = restTemplate;
    }

    public List<VirtualNode> getVirtualNodeList() {

        // Request authorization header
		HttpHeaders headers = basicAuthorizationHeader();
		
		// Request
		final HttpEntity<String> request = new HttpEntity<>(headers);

        ResponseEntity<VirtualNodeList> resp = this.restTemplate.exchange(
            "https://localhost:8888/api/v1/virtualnode",
            HttpMethod.GET, 
            request, 
            VirtualNodeList.class );

        if (resp.getStatusCode() != HttpStatus.OK) {
            throw new RuntimeException("CordaClient.getVirtualNodeList: unexpected request status "+resp.getStatusCode()) ;
        }

        return requireNonNull(resp.getBody(), "CordaClient.getVirtualNodeList: empty getBody()")
            .virtualNodes();
    }

    /**
     * Obtain list of unconsumed (active) GameProposals 
     * @param holdingIdentity
     * @return GameProposals list in JSON form
     * @throws JsonProcessingException
     */
    public String listGameProposals(HoldingIdentity holdingIdentity) throws JsonProcessingException {
        // Request authorization header
		HttpHeaders headers = basicAuthorizationHeader();

        
        
        RequestBody body = new RequestBody("list-2", "djmil.cordacheckers.gameproposal.ListFlow", new Empty());
        
        ObjectMapper mapper = new ObjectMapper();

        String json =  mapper.writeValueAsString(body);

        if (json != null)
            return json;
        // new String("{\n" + //
        //         "  \"clientRequestId\": \"list-1\",\n" + //
        //         "  \"flowClassName\": \"djmil.cordacheckers.gameproposal.ListFlow\",\n" + //
        //         "  \"requestBody\": {}\n" + //
        //         "}");

		System.out.println("HiH "+holdingIdentity.shortHash()+"\nRequst JSON = "+json);

 
		// Request
		final HttpEntity<String> request = new HttpEntity<>(json, headers);

        ResponseEntity<String> resp = this.restTemplate.exchange(
            "https://localhost:8888/api/v1/flow/"+holdingIdentity.shortHash(),
            HttpMethod.POST,
            request, 
            String.class);
        
        return resp.getBody();
    }

    private HttpHeaders basicAuthorizationHeader() {
        HttpHeaders headers = new HttpHeaders();
		headers.setBasicAuth("admin", "admin");
		
        return headers;
    }
}
